@@ -2,7 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
- * Copyright (C) 2017-2020 Osimis S.A., Belgium
+ * Copyright (C) 2017-2021 Osimis S.A., Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,6 +35,7 @@
 #include <gdcmTagKeywords.h>
 #include <gdcmUIDGenerator.h>
 #include <gdcmVersion.h>
+#include <gdcmImageHelper.h>
 
 
 static OrthancPlugins::GdcmDecoderCache  cache_;
@@ -192,6 +193,23 @@ static void AnswerTranscoded(OrthancPluginMemoryBuffer* transcoded /* out */,
                              const gdcm::Image&         image,
                              const gdcm::ImageReader&   reader)
 {
+  /**
+   * In GDCM, if "ForceRescaleInterceptSlope" is "false" (the default
+   * value), the SOP Class UID (0008,0016) might be changed from
+   * 1.2.840.10008.5.1.4.1.1.4 (MR Image Storage) to
+   * 1.2.840.10008.5.1.4.1.1.4.1 (Enhanced MR Image Storage), because
+   * of function "ImageHelper::ComputeMediaStorageFromModality()" that
+   * is called by "ImageWriter::ComputeTargetMediaStorage()". But,
+   * changing the SOP Class UID is unexpected if doing transcoding.
+   *
+   * As another side-effect, the DICOM tags "ImagePositionPatient"
+   * (0020,0032) and "ImageOrientationPatient" (0020,0037) are removed
+   * from the root of the dataset, and moved into subsequence "Shared
+   * Functional Groups Sequence" (5200,9229). This leads to issue
+   * LSD-598.
+   **/
+  gdcm::ImageHelper::SetForceRescaleInterceptSlope(true);
+
   gdcm::ImageWriter writer;
   writer.SetImage(image);
   writer.SetFile(reader.GetFile());
@@ -322,16 +340,24 @@ OrthancPluginErrorCode TranscoderCallback(
       gdcm::TransferSyntax syntax(gdcm::TransferSyntax::GetTSType(allowedSyntaxes[i]));
       if (syntax.IsValid())
       {
+        if (reader.GetImage().GetPixelFormat().GetBitsAllocated() == 1u)
+        {
+          // Prevent transcoding of 1-bit images, as this might crash GDCM
+          // https://groups.google.com/g/orthanc-users/c/xIwrkFRceuE/m/jwxy50djAQAJ
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented, "Cannot transcode 1bpp DICOM images");
+        }
+        
         if (reader.GetImage().GetPixelFormat().GetBitsStored() == 16u &&
             syntax == gdcm::TransferSyntax::JPEGExtendedProcess2_4)
         {
           /**
-           * This is a temporary workaround for issue #513 in GDCM:
+           * This is a temporary workaround for issue #513 in GDCM
+           * (will be fixed in GDCM 3.0.9):
            * https://sourceforge.net/p/gdcm/bugs/513/
            * https://groups.google.com/g/orthanc-users/c/xt9hwpj6mlQ
            **/
           throw Orthanc::OrthancException(Orthanc::ErrorCode_NotImplemented,
-                                          "Transcoding 16bit images to 1.2.840.10008.1.2.4.51 might lead to a crash in GDCM");
+                                          "Transcoding 16bpp images to 1.2.840.10008.1.2.4.51 might lead to a crash in GDCM");
         }
         
         gdcm::ImageChangeTransferSyntax change;
